package com.perfectech.tis.activities;

import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.database.sqlite.SQLiteException;
import android.graphics.Color;
import android.support.design.widget.TabLayout;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;

import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.os.Bundle;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;

import android.widget.TextView;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.R;
import com.perfectech.tis.TISDBHelper;
import com.perfectech.tis.syncClasses.TISCommon;
import com.perfectech.tis.webServices.SyncJson;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;

public class CreateJob extends AppCompatActivity
        implements CreateJobPickupFragment.OnDataPass, CreateJobMainFragment.OnDataPass, CreateJobDeliveryFragment.OnDataPass {

    private SectionsPagerAdapter mSectionsPagerAdapter;

    private ViewPager mViewPager;
    private Bundle bMain;
    private Bundle bPickup;
    private Bundle bDeliver;
    private Integer iActiveFragment;
    private static String sUserID = "";
    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 6;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_create_job);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        // Create the adapter that will return a fragment for each of the three
        // primary sections of the activity.
        mSectionsPagerAdapter = new SectionsPagerAdapter(getSupportFragmentManager());
        bMain = new Bundle();
        bPickup = new Bundle();
        bDeliver = new Bundle();
        iActiveFragment = 0; //We always start with the first fragment

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        sUserID = prefs.getString("UserID", "No Value");

        // Set up the ViewPager with the sections adapter.
        mViewPager = (ViewPager) findViewById(R.id.container);
        mViewPager.setAdapter(mSectionsPagerAdapter);
        mViewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

            }

            @Override
            public void onPageSelected(int position) {
                iActiveFragment = position;
            }

            @Override
            public void onPageScrollStateChanged(int state) {

            }
        });

        TabLayout tabLayout = (TabLayout) findViewById(R.id.tabs);
        tabLayout.setBackgroundColor(Color.DKGRAY);
        tabLayout.setupWithViewPager(mViewPager);
        downloadCreateJobItems();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_create_job, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.cj_action_save) {
            saveJob();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    public class SectionsPagerAdapter extends FragmentPagerAdapter {

        public SectionsPagerAdapter(FragmentManager fm) {
            super(fm);
        }

        @Override
        public Fragment getItem(int position) {
            // getItem is called to instantiate the fragment for the given page.
            // Return a PlaceholderFragment (defined as a static inner class below).
            switch (position) {
                case 0:
                    return CreateJobMainFragment.newInstance();
                case 1:
                    return CreateJobPickupFragment.newInstance();
                case 2:
                    return CreateJobDeliveryFragment.newInstance();
                default:
                    return CreateJobMainFragment.newInstance();
            }
        }

        @Override
        public int getCount() {
            // Show 3 total pages.
            return 3;
        }

        @Override
        public CharSequence getPageTitle(int position) {
            switch (position) {
                case 0:
                    return getResources().getString(R.string.job_information);
                case 1:
                    return getResources().getString(R.string.pickup_information);
                case 2:
                    return getResources().getString(R.string.delivery_information);
            }
            return null;
        }
    }

    public void onDataPass(String sFragmentName, Bundle data) {
        switch (sFragmentName.toLowerCase()) {
            case "main":
                bMain = data;
                break;
            case "pickup":
                bPickup = data;
                break;
            case "deliver":
                bDeliver = data;
                break;
        }
    }

    private void saveJob() {
        //This is cheesy but the only way I know to do this right now.
        //Moving off of current fragment to save current fragment data and then back to the
        //current fragment
        if (iActiveFragment == 0) {
            mViewPager.setCurrentItem(3);
        } else {
            mViewPager.setCurrentItem(0);
        }
        if (validInput()) {
            JSONObject oJobInfo = new JSONObject();
            try {
                oJobInfo.put("userID", sUserID);
                oJobInfo.put("jobMain", formatJobMainInfo());
                oJobInfo.put("jobPickup", formatJobPickupInfo());
                oJobInfo.put("jobDeliver", formatJobDeliverInfo());
                SyncJson oSyncJobs = new SyncJson();
                oSyncJobs.oContext = getApplicationContext();
                oSyncJobs.webserviceName = "insNewJob.php";
                String sJSONJobString = oJobInfo.toString();
                oSyncJobs.sendString = sJSONJobString;
                String sResponse = oSyncJobs.SyncData();
                if (sResponse.equals("success")) {
                    String sTitle = getResources().getString(R.string.create_job_success_title);
                    String sText = getResources().getString(R.string.create_job_success_text);
                    successDialog(sTitle, sText);
                } else {
                    String sTitle = getResources().getString(R.string.create_job_failed_title);
                    String sText = getResources().getString(R.string.create_job_failed_text);
                    errorDialog(sTitle, sText);
                }
            } catch (JSONException ex) {
                DBLogs.insert("JSON Error: " + ex.toString(),
                        "CreateJob.saveJob", getApplicationContext(),
                        sDBName, iVersionVal);
            }

        } else {
            String sTitle = getResources().getString(R.string.create_job_error_required_title);
            String sText = getResources().getString(R.string.create_job_error_required_text);
            errorDialog(sTitle, sText);
        }
    }

    private boolean validInput() {
        //Set variable that will define if all required fields have been entered
        Boolean bValidEntry = true;
        if (!bMain.isEmpty()) {
            if (bMain.getString("jobnumber").length() < 2) {
                bValidEntry = false;
            }
            if (bMain.getString("pickupdate").length() < 3) {
                bValidEntry = false;
            }
            if (bMain.getString("pickuptime").length() < 3) {
                bValidEntry = false;
            }
            if (bMain.getString("deliverdate").length() < 3) {
                bValidEntry = false;
            }
            if (bMain.getString("delivertime").length() < 3) {
                bValidEntry = false;
            }
        } else {
            bValidEntry = false;
        }
        if (bValidEntry && !bPickup.isEmpty()) {
            if (bPickup.getString("from").length() < 2) {
                bValidEntry = false;
            }
            if (bPickup.getString("address1").length() < 5) {
                bValidEntry = false;
            }
            if (bPickup.getString("city").length() < 2) {
                bValidEntry = false;
            }
            if (bPickup.getString("state").equals(getResources().getString(R.string.create_job_spinner_hint))) {
                bValidEntry = false;
            }
            if (bPickup.getString("postalcode").length() < 4) {
                bValidEntry = false;
            }
        } else {
            bValidEntry = false;
        }
        if (bValidEntry && !bDeliver.isEmpty()) {
            if (bDeliver.getString("to").length() < 2) {
                bValidEntry = false;
            }
            if (bDeliver.getString("address1").length() < 5) {
                bValidEntry = false;
            }
            if (bDeliver.getString("city").length() < 2) {
                bValidEntry = false;
            }
            if (bDeliver.getString("state").equals(getResources().getString(R.string.create_job_spinner_hint))) {
                bValidEntry = false;
            }
            if (bDeliver.getString("postalcode").length() < 4) {
                bValidEntry = false;
            }
        } else {
            bValidEntry = false;
        }
        return bValidEntry;
    }

    private void errorDialog(String sTitle, String sMessage) {
        TextView title = new TextView(this);
        title.setText(sTitle);
        title.setBackgroundColor(Color.RED);
        title.setPadding(10, 10, 10, 10);
        title.setGravity(Gravity.CENTER);
        title.setTextColor(Color.WHITE);
        title.setTextSize(20);
        new AlertDialog.Builder(this)
                .setCustomTitle(title)
                .setMessage(sMessage)
                .setPositiveButton(getResources().getString(R.string.ok), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        // Do nothing but close the dialog
                        dialog.dismiss();
                    }
                })
                .show();
    }

    private void successDialog(String sTitle, String sMessage) {
        TextView title = new TextView(this);
        title.setText(sTitle);
        title.setBackgroundColor(Color.GREEN);
        title.setPadding(10, 10, 10, 10);
        title.setGravity(Gravity.CENTER);
        title.setTextColor(Color.WHITE);
        title.setTextSize(20);
        new AlertDialog.Builder(this)
                .setCustomTitle(title)
                .setMessage(sMessage)
                .setPositiveButton(getResources().getString(R.string.ok), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        // Do nothing but close the dialog
                        dialog.dismiss();
                        finish();
                    }
                })
                .show();
    }

    private void downloadCreateJobItems() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                Context oContext = getApplicationContext();
                if (TISCommon.isOnline(oContext)) {
                    //Creating the hashes of all the data tables that will be passed to the
                    //webservice to see if we need a download.  This way we only download changes
                    JSONObject oTableHashes = new JSONObject();
                    TISDBHelper oDB = new TISDBHelper(oContext, sDBName, null, iVersionVal);
                    //Open a database connection and get all of the hashes
                    try {
                        oDB.createDataBase();
                        oDB.openDataBase();
                        try {
                            oTableHashes.put("countries", createMD5HashFromTable("Countries", oDB));
                            oTableHashes.put("state", createMD5HashFromTable("States", oDB));
                            oTableHashes.put("measurementTypes", createMD5HashFromTable("MeasurementTypes", oDB));
                        } catch (JSONException ex) {
                            DBLogs.insert("JSON Error: " + ex.toString(),
                                    "CreateJob.downloadCreateJobItems1", oContext,
                                    sDBName, iVersionVal);
                        }
                    } catch (IOException ioe) {
                        throw new Error("Unable to create database");
                    } catch (SQLiteException ex) {
                        DBLogs.insert("Sqlite Error: " + ex.toString(),
                                "CreateJob.downloadCreateJobItems2", oContext,
                                sDBName, iVersionVal);
                    } finally {
                        oDB.close();
                    }
                    SyncJson oCreateJobSelection = new SyncJson();
                    oCreateJobSelection.oContext = oContext;
                    //Send the tables and hash to the servers
                    oCreateJobSelection.sendString = oTableHashes.toString();
                    oCreateJobSelection.webserviceName = "getCreateJobSelectionData.php";
                    String sResponse = oCreateJobSelection.SyncData();
                    //Check to make sure something reasonable was returned
                    if (sResponse.length() > 10) {
                        try {
                            oDB.openDataBase();
                            try {
                                //Parse the return.  This is a json object that consists of
                                //the table name and the key value pair array of field name / field value
                                JSONObject oDVIRObject = new JSONObject(sResponse);
                                for (int i = 0; i < oDVIRObject.names().length(); i++) {
                                    String sTable = oDVIRObject.names().getString(i);
                                    updateTable(sTable, oDVIRObject.getJSONArray(sTable), oDB);
                                }
                            } catch (JSONException ex) {
                                DBLogs.insert("JSON Error: " + ex.toString(),
                                        "DVIRActivit.downloadDVIRItems", oContext,
                                        sDBName, iVersionVal);
                            }
                        } catch (SQLiteException ex) {
                            DBLogs.insert("Sqlite Error: " + ex.toString(),
                                    "DVIRActivity.downloadDVRItems", oContext,
                                    sDBName, iVersionVal);
                        } finally {
                            oDB.close();
                        }
                    }
                }
            }
        }).start();
    }

    //Create the MD5 hash that will be used to see if the table data is the same on the server
    private String createMD5HashFromTable(String sTable, TISDBHelper oDB) {
        String sHash = "";
        String sSQL = ("select * from " + sTable);
        ArrayList<ArrayList<String>> tableArray = oDB.selectRecordsFromDBList(sSQL, null);
        JSONArray oTableArray = new JSONArray(tableArray);
        sHash = createJsonHash(oTableArray.toString());
        return sHash;
    }

    private String createJsonHash(String sInput) {
        String sHash = "";
        try {
            MessageDigest m = MessageDigest.getInstance("MD5");
            byte[] hashedBytes = m.digest(sInput.getBytes("UTF-8"));
            sHash = convertByteArrayToHexString(hashedBytes);
        } catch (NoSuchAlgorithmException ex) {
            DBLogs.insert("MD5 Error: " + ex.toString(),
                    "DVIRActivit.createJsonHash", getApplicationContext(),
                    sDBName, iVersionVal);
        } catch (UnsupportedEncodingException ex) {
            DBLogs.insert("MD5 Encoding Error: " + ex.toString(),
                    "DVIRActivit.createJsonHash", getApplicationContext(),
                    sDBName, iVersionVal);
        }
        return sHash;
    }

    private String convertByteArrayToHexString(byte[] arrayBytes) {
        StringBuffer stringBuffer = new StringBuffer();
        for (int i = 0; i < arrayBytes.length; i++) {
            stringBuffer.append(Integer.toString((arrayBytes[i] & 0xff) + 0x100, 16)
                    .substring(1));
        }
        return stringBuffer.toString();
    }

    //If table is returned then it does not match the server table so we need to delete
    //the current table data and insert the returned values
    private void updateTable(String sTable, JSONArray oTableValues, TISDBHelper oDB) {
        switch (sTable) {
            case "countries":
                //Delete the table contents
                oDB.deleteAllRecords("Countries");
                for (int iCounter = 0; iCounter < oTableValues.length(); iCounter++) {
                    ContentValues oValues = new ContentValues();
                    try {

                        JSONObject oCountries = oTableValues.getJSONObject(iCounter);
                        oValues.put("Code", oCountries.getString("code"));
                        oValues.put("Name_en", oCountries.getString("name_en"));
                        oValues.put("Name_sp", oCountries.getString("name_sp"));
                        oValues.put("Sort", oCountries.getInt("sort"));
                        oDB.insertRecordsInDB("Countries", null, oValues);
                    } catch (JSONException ex) {
                        DBLogs.insert("JSON Error: " + sTable + " " + ex.toString(),
                                "CreateJob.updateTable.Countries", getApplicationContext(),
                                sDBName, iVersionVal);
                    }
                }
                break;
            case "state":
                oDB.deleteAllRecords("States");
                for (int iCounter = 0; iCounter < oTableValues.length(); iCounter++) {
                    ContentValues oValues = new ContentValues();
                    try {
                        JSONObject oStates = oTableValues.getJSONObject(iCounter);
                        oValues.put("_id", oStates.getInt("id"));
                        oValues.put("Name", oStates.getString("name"));
                        oValues.put("Name_en", oStates.getString("name_en"));
                        oValues.put("Name_sp", oStates.getString("name_sp"));
                        oValues.put("Abbreviation", oStates.getString("abbreviation"));
                        oValues.put("Country", oStates.getString("country"));
                        oValues.put("Sort", oStates.getInt("sort"));
                        oDB.insertRecordsInDB("States", null, oValues);
                    } catch (JSONException ex) {
                        DBLogs.insert("JSON Error: " + sTable + " " + ex.toString(),
                                "CreateJob.updateTable.States", getApplicationContext(),
                                sDBName, iVersionVal);
                    }
                }
                break;
            case "measurementTypes":
                oDB.deleteAllRecords("MeasurementTypes");
                for (int iCounter = 0; iCounter < oTableValues.length(); iCounter++) {
                    ContentValues oValues = new ContentValues();
                    try {
                        JSONObject oMeasurementTypes = oTableValues.getJSONObject(iCounter);
                        oValues.put("_id", oMeasurementTypes.getInt("id"));
                        oValues.put("Name", oMeasurementTypes.getString("name"));
                        oValues.put("Name_en", oMeasurementTypes.getString("name_en"));
                        oValues.put("Name_sp", oMeasurementTypes.getString("name_sp"));
                        oValues.put("Sort", oMeasurementTypes.getInt("sort"));
                        oDB.insertRecordsInDB("MeasurementTypes", null, oValues);
                    } catch (JSONException ex) {
                        DBLogs.insert("JSON Error: " + sTable + " " + ex.toString(),
                                "CreateJob.updateTable.States", getApplicationContext(),
                                sDBName, iVersionVal);
                    }
                }
                break;
        }
    }

    private JSONObject formatJobMainInfo() {
        JSONObject oMain = new JSONObject();
        DateConvertClass oConvert = new DateConvertClass();
        oConvert.oContext = getApplication().getApplicationContext();
        try {
            oMain.put("jobNumber", bMain.getString("jobnumber"));
            String sPickupDateTime = oConvert.LocalToGMTTimeStamp(bMain.getString("pickupdate"), bMain.getString("pickuptime"));
            oMain.put("pickupDateTime", sPickupDateTime);
            String sDeliverDateTime = oConvert.LocalToGMTTimeStamp(bMain.getString("deliverdate"), bMain.getString("delivertime"));
            oMain.put("deliverDateTime", sDeliverDateTime);
            oMain.put("customerRefNum", bMain.getString("customerrefnum"));
            oMain.put("containerNum", bMain.getString("containernum"));
            oMain.put("entryNum", bMain.getString("entrynum"));
            oMain.put("cargoWeight", bMain.getString("cargoweight"));
            oMain.put("cargoWeightType", bMain.getString("cargoweighttype"));
            oMain.put("cargoRefNotes", bMain.getString("cargorefnote"));
        } catch (JSONException ex) {
            DBLogs.insert("JSON Error: " + ex.toString(),
                    "CreateJob.formatMainPageInfo", getApplicationContext(),
                    sDBName, iVersionVal);
        }
        return oMain;
    }

    private JSONObject formatJobPickupInfo() {
        JSONObject oPickup = new JSONObject();
        try {
            oPickup.put("name", bPickup.getString("from"));
            oPickup.put("address1", bPickup.getString("address1"));
            oPickup.put("address2", bPickup.getString("address2"));
            oPickup.put("city", bPickup.getString("city"));
            oPickup.put("stateid", bPickup.getString("state"));
            oPickup.put("postalcode", bPickup.getString("postalcode"));
            oPickup.put("phonenumber", bPickup.getString("phonenumber"));
            oPickup.put("emailaddress", bPickup.getString("emailaddress"));
            oPickup.put("countryid", bPickup.getString("country"));
            oPickup.put("Pickup", bPickup.getString("Pickup"));
        } catch (JSONException ex) {
            DBLogs.insert("JSON Error: " + ex.toString(),
                    "CreateJob.formatJobPickupInfo", getApplicationContext(),
                    sDBName, iVersionVal);
        }
        return oPickup;
    }

    private JSONObject formatJobDeliverInfo() {
        JSONObject oDeliver = new JSONObject();
        try {
            oDeliver.put("name", bDeliver.getString("to"));
            oDeliver.put("address1", bDeliver.getString("address1"));
            oDeliver.put("address2", bDeliver.getString("address2"));
            oDeliver.put("city", bDeliver.getString("city"));
            oDeliver.put("stateid", bDeliver.getString("state"));
            oDeliver.put("postalcode", bDeliver.getString("postalcode"));
            oDeliver.put("phonenumber", bDeliver.getString("phonenumber"));
            oDeliver.put("emailaddress", bDeliver.getString("emailaddress"));
            oDeliver.put("countryid", bDeliver.getString("country"));
            oDeliver.put("Pickup", bDeliver.getString("Pickup"));
        } catch (JSONException ex) {
            DBLogs.insert("JSON Error: " + ex.toString(),
                    "CreateJob.formatJobDeliverInfo", getApplicationContext(),
                    sDBName, iVersionVal);
        }
        return oDeliver;
    }
}
